#include "../../src/skipstone.h"
#include <gdk/gdkx.h>
#include <Imlib2.h>

static void create_plugin(SkipStone *ss);

static Imlib_Image im=NULL;
static gint mode = 0;
static gchar *image_path=NULL;

static GtkWidget *create_config(void);
static void save_my_config(GtkWidget *my_widget);

static SkipStonePlugin plugin =
{
   "New-Imlib",
     PLUGIN_IN_TOOLBAR_START,
     create_plugin,
     create_config,
     save_my_config,
     1
};

static void on_button_click(GtkWidget *b, GdkEventButton *ev);
static void on_selection(GtkWidget *b, GtkSelectionData *data, guint time);
static void on_expose(GtkWidget *da, GdkEvent *ev);
static void on_realize(GtkWidget *da, GdkEvent *ev);


static GtkWidget *create_config(void)
{
     GtkWidget *retval, *graphical, *path, *label, *hbox;
     retval = gtk_vbox_new(FALSE, 5);
     path = gtk_entry_new();
     label = gtk_label_new("Path to Image file: ");
     graphical = gtk_check_button_new_with_label("Use Image?");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(graphical),
				  skipstone_get_config_value_as_int("New-Imlib",
										 "mode"));
     gtk_entry_set_text(GTK_ENTRY(path), skipstone_get_config_value_as_str("New-Imlib",
									   "ImagePath"));
     hbox = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), path, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(retval), graphical, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(retval), hbox, FALSE, FALSE, 0);
     
     gtk_object_set_data(GTK_OBJECT(retval), "mode", graphical);
     gtk_object_set_data(GTK_OBJECT(retval), "ImagePath", path);
	
     gtk_widget_show_all(retval);
     return retval;
}


static void save_my_config(GtkWidget *my_widget)
{
     GtkWidget *path, *graphical;
     path = gtk_object_get_data(GTK_OBJECT(my_widget), "ImagePath");
     graphical = gtk_object_get_data(GTK_OBJECT(my_widget), "mode");
     mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(graphical));
     skipstone_set_config_value_as_str("New-Imlib","ImagePath",gtk_entry_get_text(
										  GTK_ENTRY(path)));
     image_path = skipstone_get_config_value_as_str("New-Imlib","ImagePath");
     skipstone_set_config_value_as_int("New-Imlib","mode",mode);
     return;
}
	
static void on_expose(GtkWidget *da, GdkEvent *ev)
{
   if (im == NULL) return;
     imlib_context_set_drawable(GDK_WINDOW_XWINDOW(da->window));
     imlib_context_set_image(im);
     gdk_window_clear(da->window);
     imlib_render_image_on_drawable_at_size(0,0,24,24);
}

static void on_realize(GtkWidget *da, GdkEvent *ev)
{
   
   static gint done = 0;
   
   if (done) return;
   image_path=skipstone_get_config_value_as_str("New-Imlib","ImagePath");
   if (!image_path || !strcmp(image_path," ")) return;
   imlib_context_set_display(GDK_WINDOW_XDISPLAY(da->window));
   imlib_context_set_drawable(GDK_WINDOW_XWINDOW(da->window));
   imlib_context_set_visual(GDK_VISUAL_XVISUAL(gdk_window_get_visual(da->window)));
   imlib_context_set_colormap(GDK_COLORMAP_XCOLORMAP(gdk_window_get_colormap(da->window)));
   im = imlib_load_image(image_path);
   done = 1;
}


static void on_selection(GtkWidget *b, GtkSelectionData *data, guint time)
{
   if (data->data)
      skipstone_new_window(data->data);
}

static void on_button_click(GtkWidget *b, GdkEventButton *ev)
{
   
   switch (ev->button)
     {
      case 1: 
	skipstone_new_window(NULL);
	break;
      case 2: 
	gtk_selection_convert(b, GDK_SELECTION_PRIMARY,
			      GDK_SELECTION_TYPE_STRING,
			      GDK_CURRENT_TIME);
	break;
      default:
	break;
     }
   
}

static void create_plugin(SkipStone *ss)
{
   GtkWidget *button;
   
   if (!mode) 
     {
	
	button = gtk_button_new_with_label("New");
     }
   
   else
     
     {
	GtkWidget *da;
	button = gtk_button_new();
	da = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(da), 24, 24);
	gtk_signal_connect(GTK_OBJECT(da), "expose_event", 
			   GTK_SIGNAL_FUNC(on_expose), NULL);
	gtk_signal_connect(GTK_OBJECT(da), "realize", 
			   GTK_SIGNAL_FUNC(on_realize), NULL);
	gtk_container_add(GTK_CONTAINER(button), da);
     }
   
	
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   
   gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
			GTK_SIGNAL_FUNC(on_button_click),
			NULL);
   gtk_selection_add_target(button, GDK_SELECTION_PRIMARY, 
			    GDK_SELECTION_TYPE_STRING,1);
   gtk_signal_connect(GTK_OBJECT(button), "selection_received",
		      GTK_SIGNAL_FUNC(on_selection), NULL);

   gtk_widget_show_all(button);   
   skipstone_add_plugin_to_toolbar(ss, button, 
				   "Click for a new Window/Tab or Paste buffer to open a new Window/Tab and load the contents of the buffer.");
   
}

SkipStonePlugin * init_plugin(void)
{
   
   mode = skipstone_get_config_value_as_int("New-Imlib","mode");
   if (mode == -1) 
     {
	mode = 0;
	skipstone_set_config_value_as_int("New-Imlib","mode",mode);
	skipstone_set_config_value_as_str("New-Imlib","ImagePath", " ");
     }
   
   return &plugin;
}
