#include "../../src/skipstone.h"
#include "fm.xpm"
#include "google.xpm"
#include "yahoo.xpm"
/* demo plugin for SkipStone  */
/* SkipStone will look in ~/.skipstone/plugins for shared objects to load
 * and it will pass them a pointer to the skipstone struct after packing
 * the toolbar, so you can pack anything after the toolbar is packed. 
 * Please dont access anything other than skipstone->embed
 * from the struct. For info on how to use embed to do stuff, look at gtkmozembed.h
 * <muhri@muhri.net> */

/* every plugin must have an init_plugin() function that returns a pointer to
 * a SkipStonePlugin struct */


#define GOOGLE_URL "http://www.google.com/search?q="
#define FRESHMEAT_URL "http://www.freshmeat.net/search/?q="
#define YAHOO_URL "http://search.yahoo.com/bin/search?p="

static void create_plugin(SkipStone *skipstone);
static void on_entry_activate(GtkWidget *entry, SkipStone *skipstone);
static void on_button_clicked(GtkWidget *button, SkipStone *skipstone);
static GtkWidget *get_pixmap(GtkWidget *hbox, gint i);

static SkipStonePlugin plugin =
{
	"SearchToolbar!", /* plugin name */
       	PLUGIN_AFTER_TOOLBAR, /* position after toolbar */
	 create_plugin,/* create plugin function */
       NULL,	/* get_config_widget() */
       NULL,	/* save_config() */
	  1 /* do we create this plugin for each window */
};

static void create_plugin(SkipStone *skipstone)
{
	GtkWidget *box, *hb;
	gchar *search_engines[] = { 
		  "Google!",
		  "Freshmeat!" ,
		  "Yahoo!",
		  NULL};
	gint i;
	box = gtk_hbox_new(0,5);
	hb = gtk_handle_box_new();
	for (i = 0; search_engines[i] != NULL; ++i) {
		GtkWidget *entry, *hbox, *pix, *button;
		entry  = gtk_entry_new();
	     	hbox = gtk_hbox_new(0,5);	
	     	pix = get_pixmap(hbox,i);
		
	     	if (pix) {
		     button = gtk_button_new();
		     gtk_container_add(GTK_CONTAINER(button), pix);
		} else {
		     button = gtk_button_new_with_label(search_engines[i]);
		}
		gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);	     	
		gtk_box_pack_start(GTK_BOX(hbox), button, 0, 0, 0);
		gtk_box_pack_start(GTK_BOX(hbox), entry, 0, 0, 0);
	     	gtk_object_set_data(GTK_OBJECT(button), "engine", search_engines[i]);
		gtk_object_set_data(GTK_OBJECT(entry), "engine", search_engines[i]);
		gtk_box_pack_start(GTK_BOX(box), hbox, 0, 0, 2);
		gtk_signal_connect(GTK_OBJECT(entry), "activate", 
				   GTK_SIGNAL_FUNC(on_entry_activate), 
				   skipstone);
	     	gtk_signal_connect(GTK_OBJECT(button), "clicked",
				   GTK_SIGNAL_FUNC(on_button_clicked),
				   skipstone);
	}
	
	/* pack our box onto skipstone's vbox and show it all */
	gtk_container_add(GTK_CONTAINER(hb), box);
     	skipstone_pack_plugin(skipstone, hb);
	gtk_widget_show_all(box);
}

static void on_button_clicked(GtkWidget *button, SkipStone *skipstone)
{
     	gchar *engine;
	gchar *url = NULL;
     
     	g_return_if_fail(skipstone != NULL);
     	g_return_if_fail(skipstone->embed != NULL);
     
	engine = gtk_object_get_data(GTK_OBJECT(button), "engine");
	if (!strcmp(engine, "Google!")) {
		url = "http://www.google.com";
	} else if (!strcmp(engine,"Freshmeat!")) {
		url = "http://www.freshmeat.net";
	} else if (!strcmp(engine,"Yahoo!")) {
		url = "http://www.yahoo.com";
	}
	if (url) 		   
		skipstone_load_url(skipstone, url);
	return;
}

static void on_entry_activate(GtkWidget *entry, SkipStone *skipstone)
{
	gchar *stuff, *engine;
	gchar *url = NULL;
     
     	g_return_if_fail(skipstone != NULL);
     	g_return_if_fail(skipstone->embed != NULL);
     
	stuff = gtk_entry_get_text(GTK_ENTRY(entry));
	engine = gtk_object_get_data(GTK_OBJECT(entry), "engine");
	if (!strcmp(engine, "Google!")) {
		url = g_strconcat(GOOGLE_URL,stuff,NULL);
	} else if (!strcmp(engine,"Freshmeat!")) {
		url = g_strconcat(FRESHMEAT_URL,stuff,NULL);
	} else if (!strcmp(engine,"Yahoo!")) {
		url = g_strconcat(YAHOO_URL,stuff, NULL);
	}
	if (url) {		   
		skipstone_load_url(skipstone, url);
		g_free(url);
	}
	return;
}

SkipStonePlugin *init_plugin(void)
{

	return &plugin;
}

static GtkWidget * get_pixmap(GtkWidget *hbox, gint i)
{
     GdkBitmap *mask;
     GdkPixmap *pixmap;
     GtkWidget *retpixmap = NULL;
     gchar **icon = NULL;
     
     if (!i) icon = google_xpm;
     if (i == 1) icon = fm_xpm;
     if (i == 2) icon = yahoo_xpm;
       
	
     pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
						    gtk_widget_get_colormap(hbox),
						   &mask,
						   NULL,
						   icon);

     if (pixmap && mask) {
	  	retpixmap =  gtk_pixmap_new(pixmap,mask);
	  	gdk_pixmap_unref(pixmap);
	  	gdk_bitmap_unref(mask);
     }

     return retpixmap;
}
