/* copyright 	Maher Awamy <muhri@muhri.net>		*
 *		Charlie Schmidt <ishamael@themes.org>	*
 *							*
 *  released under the GPL				*/

#include "skipstone.h"

static void on_menu_destroy(GtkWidget *menu, gchar *name);

static void on_menu_destroy(GtkWidget *menu, gchar *name)
{
     gchar *free_href, *free_img, *free_linktext;
     D_ENTER;
     free_href = gtk_object_get_data(GTK_OBJECT(menu), "freehref");
     if (free_href) g_free(free_href);
     free_img = gtk_object_get_data(GTK_OBJECT(menu), "freeimg");
     if (free_img) g_free(free_img);
     free_linktext = gtk_object_get_data(GTK_OBJECT(menu), "freelinktext");
     if (free_linktext) g_free(free_linktext);
     D_RETURN;
}


GtkWidget * get_link_popup_menu(SkipStone *skipstone, gchar *href, gchar *linktext)
{
     	GtkWidget *menu, *m1,*m2,*m3, *m4, *m5;
     	D_ENTER;
     	if (skipstone->link_menu) gtk_widget_destroy(skipstone->link_menu);
     	menu = gtk_menu_new();
     	gtk_object_set_data(GTK_OBJECT(menu), "freehref", href);
     	gtk_object_set_data(GTK_OBJECT(menu), "freelinktext", linktext);
     	gtk_signal_connect(GTK_OBJECT(menu), "destroy", on_menu_destroy, "link menu");
     	m1 = gtk_menu_item_new_with_label(_("Open in this Window"));
     	gtk_signal_connect(GTK_OBJECT(m1), "activate", GTK_SIGNAL_FUNC(browse_in_window_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m1), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m1);
     	m2 = gtk_menu_item_new_with_label(_("Open in a new Window"));
     	gtk_menu_append(GTK_MENU(menu), m2);
     	gtk_object_set_data(GTK_OBJECT(m2), "link", href);
     	gtk_signal_connect(GTK_OBJECT(m2), "activate", GTK_SIGNAL_FUNC(make_window_from_popup), skipstone);
     	m3  = gtk_menu_item_new_with_label(_("Download Link..."));
     	gtk_signal_connect(GTK_OBJECT(m3), "activate", GTK_SIGNAL_FUNC(download_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m3), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m3);
     	m4 = gtk_menu_item_new_with_label(_("Copy Link location"));
     	gtk_signal_connect(GTK_OBJECT(m4), "activate", GTK_SIGNAL_FUNC(copy_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m4), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m4);
     	m5 = gtk_menu_item_new_with_label(_("Add bookmark for link"));
     	gtk_object_set_data(GTK_OBJECT(m5), "link", href);
     	gtk_object_set_data(GTK_OBJECT(m5), "linktext", linktext);
     	gtk_signal_connect(GTK_OBJECT(m5), "activate", GTK_SIGNAL_FUNC(add_link_from_popup), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m5);
     	gtk_widget_show_all(menu);
     	skipstone->link_menu = menu;
     	D_RETURN_ (menu);
             
}

GtkWidget * get_image_popup_menu(SkipStone *skipstone, gchar *img)
{
     	GtkWidget *menu, *m1,*m2,*m3, *m4;
     	gchar *lbl, *img_basename;
     
     	D_ENTER;
     
     	if (skipstone->image_menu) gtk_widget_destroy(skipstone->image_menu);
     	img_basename = g_basename(img);
     	menu = gtk_menu_new();
     	gtk_object_set_data(GTK_OBJECT(menu), "freeimg", img);
     	gtk_signal_connect(GTK_OBJECT(menu), "destroy", on_menu_destroy, "image menu");
     	m1 = gtk_menu_item_new_with_label(_("Open Image in this Window"));
     	gtk_signal_connect(GTK_OBJECT(m1), "activate", GTK_SIGNAL_FUNC(browse_in_window_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m1), "link", img);
     	gtk_menu_append(GTK_MENU(menu), m1);
     	m2 = gtk_menu_item_new_with_label(_("Open Image in a new Window"));
     	gtk_menu_append(GTK_MENU(menu), m2);
     	gtk_object_set_data(GTK_OBJECT(m2), "link", img);
     	gtk_signal_connect(GTK_OBJECT(m2), "activate", GTK_SIGNAL_FUNC(make_window_from_popup), skipstone);
     	lbl = g_strdup_printf("%s... (%s)",_("Save Image"),img_basename);
     	m3  = gtk_menu_item_new_with_label(lbl);
     	gtk_signal_connect(GTK_OBJECT(m3), "activate", GTK_SIGNAL_FUNC(save_image_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m3), "link", img);
     	gtk_object_set_data(GTK_OBJECT(m3), "name", img_basename);
     	gtk_menu_append(GTK_MENU(menu), m3);
     	m4 = gtk_menu_item_new_with_label(_("Copy Image location"));	
     	gtk_signal_connect(GTK_OBJECT(m4), "activate", GTK_SIGNAL_FUNC(copy_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m4), "link", img);
     	gtk_menu_append(GTK_MENU(menu), m4);
     	gtk_widget_show_all(menu);
     	g_free(lbl);
     	skipstone->image_menu = menu;
     	D_RETURN_(menu);
}


GtkWidget * get_doc_popup_menu(SkipStone *skipstone, gchar *href)
{
     	GtkWidget *menu, *m1,*m2,*m3, *m4, *m5, *m6, *m7;
     	D_ENTER;
     	if (skipstone->doc_menu) gtk_widget_destroy(skipstone->doc_menu);
     	menu = gtk_menu_new();
     	gtk_object_set_data(GTK_OBJECT(menu), "freehref", href);
     	gtk_signal_connect(GTK_OBJECT(menu), "destroy", on_menu_destroy, "doc menu"									   );
     	m1 = gtk_menu_item_new_with_label(_("Back"));
     	gtk_signal_connect(GTK_OBJECT(m1), "activate", GTK_SIGNAL_FUNC(on_back), skipstone);
     	gtk_widget_set_sensitive(m1, gtk_moz_embed_can_go_back(skipstone->embed));
     
     	gtk_menu_append(GTK_MENU(menu), m1);
     	m2 = gtk_menu_item_new_with_label(_("Forward"));	
     	gtk_menu_append(GTK_MENU(menu), m2);
     	gtk_signal_connect(GTK_OBJECT(m2), "activate", GTK_SIGNAL_FUNC(on_forward), skipstone);
     	gtk_widget_set_sensitive(m2, gtk_moz_embed_can_go_forward(skipstone->embed));
          	          		     	
     	m3  = gtk_menu_item_new_with_label(_("Reload"));	
     	gtk_signal_connect(GTK_OBJECT(m3), "activate", GTK_SIGNAL_FUNC(on_reload), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m3);

     	m4 = gtk_menu_item_new_with_label(_("Stop"));
     	gtk_signal_connect(GTK_OBJECT(m4), "activate", GTK_SIGNAL_FUNC(on_stop), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m4);
     
     	m5 = gtk_menu_item_new_with_label(_("Find"));
     	gtk_signal_connect(GTK_OBJECT(m5), "activate", GTK_SIGNAL_FUNC(find_dialog), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m5);
     
     	m6 = gtk_menu_item_new_with_label(_("Add Bookmark"));
     	gtk_signal_connect(GTK_OBJECT(m6), "activate", GTK_SIGNAL_FUNC(add_bookmark), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m6);
     
     	m7 = gtk_menu_item_new_with_label(_("View Source"));	
     	gtk_signal_connect(GTK_OBJECT(m7), "activate", GTK_SIGNAL_FUNC(on_view_source), skipstone);
     	gtk_menu_append(GTK_MENU(menu), m7);     
     
     	gtk_widget_show_all(menu);
     	skipstone->doc_menu = menu;
     	D_RETURN_ ( menu );
             
}

GtkWidget * get_link_image_popup_menu(SkipStone *skipstone, gchar *img, gchar *href)
{
     	GtkWidget *menu, *m1,*m2,*m3, *m4, *sep, *m5, *m6, *m7, *m8;
     	gchar *lbl, *img_basename;
     	D_ENTER;
     	if (skipstone->image_link_menu) gtk_widget_destroy(skipstone->image_link_menu);
     	img_basename = g_basename(img);
     	menu = gtk_menu_new();
     	gtk_object_set_data(GTK_OBJECT(menu), "freehref", href);
     	gtk_object_set_data(GTK_OBJECT(menu), "freeimg", img);
     	gtk_signal_connect(GTK_OBJECT(menu), "destroy", on_menu_destroy, "link image menu");
     	m1 = gtk_menu_item_new_with_label(_("Open Image in this Window"));
     	gtk_signal_connect(GTK_OBJECT(m1), "activate", GTK_SIGNAL_FUNC(browse_in_window_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m1), "link", img);	     
     	gtk_menu_append(GTK_MENU(menu), m1);
             
     	m2 = gtk_menu_item_new_with_label(_("Open Image in a new Window"));
     	gtk_menu_append(GTK_MENU(menu), m2);
     	gtk_object_set_data(GTK_OBJECT(m2), "link", img);
     	gtk_signal_connect(GTK_OBJECT(m2), "activate", GTK_SIGNAL_FUNC(make_window_from_popup), skipstone);
     	
     	lbl = g_strdup_printf("%s... (%s)",_("Save Image"),img_basename);
     	m3  = gtk_menu_item_new_with_label(lbl);
     	gtk_signal_connect(GTK_OBJECT(m3), "activate", GTK_SIGNAL_FUNC(save_image_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m3), "link", img);
     	gtk_object_set_data(GTK_OBJECT(m3), "name", img_basename);
     	gtk_menu_append(GTK_MENU(menu), m3);
     	     
     	m4 = gtk_menu_item_new_with_label(_("Copy Image location"));
     	gtk_signal_connect(GTK_OBJECT(m4), "activate", GTK_SIGNAL_FUNC(copy_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m4), "link", img);
     	gtk_menu_append(GTK_MENU(menu), m4);

     	sep = gtk_menu_item_new();
     	gtk_menu_append(GTK_MENU(menu), sep);
     
     	m5 = gtk_menu_item_new_with_label(_("Open in this Window"));
     	gtk_signal_connect(GTK_OBJECT(m5), "activate", GTK_SIGNAL_FUNC(browse_in_window_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m5), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m5);
             
     	m6 = gtk_menu_item_new_with_label(_("Open in a new Window"));
     	gtk_menu_append(GTK_MENU(menu), m6);
     	gtk_object_set_data(GTK_OBJECT(m6), "link", href);
     	gtk_signal_connect(GTK_OBJECT(m6), "activate", GTK_SIGNAL_FUNC(make_window_from_popup), skipstone);

     	m7 = gtk_menu_item_new_with_label(_("Download Link"));
     	gtk_signal_connect(GTK_OBJECT(m7), "activate", GTK_SIGNAL_FUNC(download_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m7), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m7);
     
     	m8 = gtk_menu_item_new_with_label(_("Copy Link location"));
     	gtk_signal_connect(GTK_OBJECT(m8), "activate", GTK_SIGNAL_FUNC(copy_link_from_popup), skipstone);
     	gtk_object_set_data(GTK_OBJECT(m8), "link", href);
     	gtk_menu_append(GTK_MENU(menu), m8);
     
     	gtk_widget_show_all(menu);
     	skipstone->image_link_menu = menu;
     	D_RETURN_ ( menu );
}

GtkWidget * get_reload_menu(SkipStone *skipstone)
{
	GtkWidget *m1,*m2,*m3, *m4;
     	D_ENTER;
	if (skipstone->reload_menu != NULL) D_RETURN_ (skipstone->reload_menu);
	
	skipstone->reload_menu = gtk_menu_new();
	m1 = gtk_menu_item_new_with_label(_("Reload Normal"));
	gtk_signal_connect(GTK_OBJECT(m1), "activate", GTK_SIGNAL_FUNC(on_reload_popup_menu_selection_made), skipstone);
	gtk_object_set_data(GTK_OBJECT(m1), "reload_flag", GINT_TO_POINTER(0));
	gtk_menu_append(GTK_MENU(skipstone->reload_menu), m1);
	
	m2 = gtk_menu_item_new_with_label(_("Reload by passing Cache (shift - click)"));
	gtk_menu_append(GTK_MENU(skipstone->reload_menu), m2);
	gtk_object_set_data(GTK_OBJECT(m2), "reload_flag", GINT_TO_POINTER(1));
	gtk_signal_connect(GTK_OBJECT(m2), "activate", GTK_SIGNAL_FUNC(on_reload_popup_menu_selection_made), skipstone);
	
	m3  = gtk_menu_item_new_with_label(_("Reload by passing Proxy"));
	gtk_signal_connect(GTK_OBJECT(m3), "activate", GTK_SIGNAL_FUNC(on_reload_popup_menu_selection_made), skipstone);
	gtk_object_set_data(GTK_OBJECT(m3), "reload_flag", GINT_TO_POINTER(2));
	gtk_menu_append(GTK_MENU(skipstone->reload_menu), m3);
	
	m4 = gtk_menu_item_new_with_label(_("Reload by passing Proxy & Cache"));
	gtk_signal_connect(GTK_OBJECT(m4), "activate", GTK_SIGNAL_FUNC(on_reload_popup_menu_selection_made), skipstone);
	gtk_object_set_data(GTK_OBJECT(m4), "reload_flag", GINT_TO_POINTER(3));
	gtk_menu_append(GTK_MENU(skipstone->reload_menu), m4);
	
	gtk_widget_show_all(skipstone->reload_menu);
	D_RETURN_(skipstone->reload_menu);
             
}
